use std::collections::HashMap;

use super::{AccountName, UnitName};

#[derive(Debug, serde::Deserialize, PartialEq)]
#[serde(deny_unknown_fields, rename_all = "snake_case")]
pub enum CSVColumnSpec {
    Ignore,
    Datestamp,
    Title,
    Deposit,
    Withdraw,
    Change,
    Balance,
    Unit,
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
pub struct CSVImportSpec {
    pub skip_start: Option<usize>,
    pub skip_end: Option<usize>,
    pub cols: Vec<CSVColumnSpec>,
    pub date_format: String,
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields, tag = "format")]
pub enum ImportFileFormat {
    CSV(CSVImportSpec),
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
pub struct AccountSpec {
    pub title: Option<String>,
    pub description: Option<String>,

    pub annotations: Option<HashMap<String, String>>,

    pub unit: Option<UnitName>,

    pub import: Option<ImportFileFormat>,
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
pub struct UnitSpec {
    pub name: UnitName,
    pub precision: Option<u32>,
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
pub struct SpecRoot {
    pub ledger_path: std::path::PathBuf,

    pub units: HashMap<UnitName, UnitSpec>,

    pub accounts: HashMap<AccountName, AccountSpec>,
}
